﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Globalization;
using System.Reflection;
using System.Text;
using Produmex.Foundation;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.DbClient;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Convertors;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.Generators.BarcodeController;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.DataObjects;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Convertors;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Generators.SerialNumberController;
using Produmex.Sbo.Logex.Data.Providers;


// PmxWorkflowExecutionTypes.SUB_FLOW
namespace Produmex.Sbo.Logex.WorkflowScripts
{
    /// <summary>
    /// Replenish script
	/// Customized: Changed order by for move order lines: Priority, due date, location sequence
    /// </summary>
    public class WorkflowScript_ReplenishScript : WorkflowInstanceScriptBase
    {

        private string current_SSCC;
        private string current_ItemDescription;
        private string current_SourceLocationCode;
        private string current_DestinationLocationCode;
        private int? current_LUID = null;
        private double current_OpenQuantity;
        private string current_Uom;

        private ProdumexError m_pmxError;



        // Input parameters
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;

        private CultureInfo current_DefaultCultureInfo
        {
            get { return DefaultCultureInfo.Get(); }
        }

        private PmxOseCompany currentPmxOseCompany
        {
            get { return PmxOseCompany.Get(); }
        }


        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);


        #region Protected Members

        protected string current_Batchnumber;
        protected DateTime? current_BestBeforeDate;// = null;
        protected string current_ItemCode;
        protected int? current_ItemTransactionalInfoKey;
        protected string current_QualityStatusCode;

        protected bool m_subBackRequested;// = false;
        protected string m_sourceLocationCode; // = null;
        protected string m_deviceID;
        protected string m_subInitialErrorKey;
        protected string m_itemCode;
        protected DateTime? m_bestBeforeDate;
        protected string m_qualityStatusCode;
        protected string m_SSCC;
        protected double m_maximumQuantity;
        protected string m_itemInventoryUom;
        protected int? m_itemTransactionalInfoKey;
        protected string m_batchNumber;

        protected PmxItemInfo m_itemInfo;



        // Sub control
        protected bool SubBackRequested;
        protected double SubQuantity;


        protected Collection<string> m_subFlowListOfSerialNumbers;

        WorkflowInstanceBase m_waProcessingSubFlow;

        // Sub control
        protected string m_subFlowActualDestinationLocationCode;
        protected List<MoveItemInfo> m_subFlowActualMoveItemInfoList;


        #endregion


        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_ReplenishScript"/> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_ReplenishScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
            m_waProcessingSubFlow = this.Create("WAMoveProcessingScript");

        }

        #endregion

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

            SboUserInfo userInfo = sboProviderService.GetCurrentUser();
            Message msg = null;
            string buttonKey = null;
            string query = null;
            string initialErrorKey = null;
            Dictionary<string, PmxStorageLocationCanItemBeStoredInfo> itemQuantityItriKeysBatchNumsPerItemCode = null;

            int? MODocEntry = null;
            int? MOLineID = null;
            string recieved_SSCC = null;
            double recieved_Quantity = 0;
            string destinationLocationCode = null;

            double maximumQuantityForLocation = 0;

            bool isSourceLocationLocked = false;
            bool isDestinationLocationLocked = false;
            bool isFullMonoPallet = false;
            bool canItemBeStoredOnLocation = false;

            DataSet dsMoveOrders = null;
            DataSet dsSourceLocations = null;
            DataSet dsDestinationLocations = null;
            DataSet dsProductsOnSSCC = null;
            DataSet dsLuidFromStock = null;
            DataSet ds = null;

            Collection<MoveItemInfo> moveItems = null;
            Collection<string> itemCodes = null;
            Collection<PmxItemInfo> itemInfos = null;

            Collection<string> listOfSerialNumbers = null;
            bool askForSerialNumbers = false;
            bool forceSerial = false;
            bool useRange = false;
            bool askReprint = false;

            PmxOseWarehouseInfo whsInfoDevice = sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), session.DeviceInstanceId);

            PmxOseWarehouseInfo whsInfo = null;

            ProdumexError pmxError = null;
            int numberOfReports = 0;

            IBarcodeController barcodeController = null;
            Collection<string> listOfLocationsForWA = new Collection<string>();

            sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
                delegate(PmxDbConnection conn, object[] parameters)
                {
                    PmxBarcodeProvider bcProv = new PmxBarcodeProvider( conn );
                    barcodeController = bcProv.GetBarcodeController();

                    PmxOseWarehouseAutomationProvider prov = new PmxOseWarehouseAutomationProvider( conn );
                    listOfLocationsForWA = prov.GetAllInputLocationsForWA(true);

                    return null;
                });


            #region Declaraction SubFlow Select Product From Stock flow - MEMBERS

            WorkflowInstanceBase subSelectProductFromStockFlow = null;

            current_Batchnumber = null;
            current_BestBeforeDate = null;// = null;
            current_ItemCode = null;
            current_ItemTransactionalInfoKey = null;
            current_QualityStatusCode = null;

            m_subBackRequested = false;
            m_sourceLocationCode = null; // = null;
            m_deviceID = null;
            initialErrorKey = null;
            m_itemCode = null;
            m_bestBeforeDate = null;
            m_qualityStatusCode = null;
            m_SSCC = null;
            m_maximumQuantity = 0;
            m_itemInventoryUom = null;
            m_itemTransactionalInfoKey = null;
            m_subInitialErrorKey = null;


            #endregion

        Step_SelectMoveOrderStart:

            initialErrorKey = null;
            m_deviceID = session.DeviceInstanceId;
            whsInfo = whsInfoDevice;

        Step_SelectMoveOrder:

            #region SELECT MOVE ORDER SCREEN

            query = BuildQuery.GetMoveOrdersOrdersByPriorityDueDate(null, null, null, null, PmxMoveOrderType.Replenish, true, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Inventory, whsInfo.PmxWarehouseCode, m_deviceID, sboProviderService.GetDbTool());
            dsMoveOrders = sboProviderService.RunView(false, null, null, query);

            if (GetNumberOfRows(dsMoveOrders).Equals(0))
            {
                initialErrorKey = "MSG_ERROR_NO_REPLENISH_FOUND";

                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                       DefaultCultureInfo.Get(), BuildParamCollection(
                           "MessageKey", (initialErrorKey),
                           "ShowButton", true));
                WaitForMessage();

                return;

            }

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.MoveOrder.ISelectMOScreen),
                               DefaultCultureInfo.Get(), BuildParamCollection(
                                   "MODS", dsMoveOrders,
                                   "TitleKey", "MSG_TITLE_SELECT_REPLENISH",
                                   "InitialErrorKey", initialErrorKey,
                                   "EnabledToolbarButtons", -1
                                   ));

            msg = WaitForMessage();

            if (msg.Name.EndsWith("MOSelected"))
            {
                MOLineID = ExtractParameter<int>(msg.Parameters, "lineID");
                MODocEntry = ExtractParameter<int>(msg.Parameters, "moDocEntry");

                if (GetNumberOfRows(dsMoveOrders) > 0)
                {
                    DataRow[] rows = dsMoveOrders.Tables[0].Select(MoveOrderDataSet.Columns.MO_DOC_ENTRY.Name + " = " + MODocEntry.Value.ToString() + " AND " + MoveOrderDataSet.Columns.LINEID.Name + " = " + MOLineID.Value.ToString());
                    if (rows.Length > 0)
                    {
                        current_Batchnumber = rows[0][MoveOrderDataSet.Columns.BATCHNUMBER.Name] as string;
                        current_SourceLocationCode = rows[0][MoveOrderDataSet.Columns.SOURCE_STORAGE_LOCATION.Name] as string;
                        current_ItemCode = rows[0][MoveOrderDataSet.Columns.ITEMCODE.Name] as string;

                        //GETDATA
                        m_itemInfo = (PmxItemInfo)sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
                          delegate(PmxDbConnection conn, object[] paramlist)
                          {
                              PmxItemAllConnectionsProvider prov = new PmxItemAllConnectionsProvider(conn);
                              return prov.GetCachedItemInfo(current_ItemCode);
                          });


                        current_ItemDescription = m_itemInfo.CustomItemDescription;
                        current_QualityStatusCode = rows[0][MoveOrderDataSet.Columns.SOURCE_QUALITY_STATUS_CODE.Name] as string;
                        current_SSCC = rows[0][MoveOrderDataSet.Columns.SSCC.Name] as string;
                        current_LUID = Convertor.ConvertToInt32Nullable(rows[0][MoveOrderDataSet.Columns.LUID.Name]);
                        current_ItemTransactionalInfoKey = Convertor.ConvertToInt32Nullable(rows[0][MoveOrderDataSet.Columns.ITEMTRANSACTIONALINFOKEY.Name]);
                        current_DestinationLocationCode = rows[0][MoveOrderDataSet.Columns.DESTINATION_STORAGE_LOCATION.Name] as string;

                        current_BestBeforeDate = (rows[0][MoveOrderDataSet.Columns.BESTBEFOREDATE.Name] is DateTime ? (DateTime?)(rows[0][MoveOrderDataSet.Columns.BESTBEFOREDATE.Name]) : null);

                        current_OpenQuantity = Convert.ToDouble(rows[0][MoveOrderDataSet.Columns.OPENQTY.Name]);
                        current_Uom = Convert.ToString(rows[0][MoveOrderDataSet.Columns.UOM.Name]);
                    }
                }
            }
            else if (msg.Name.EndsWith("BackRequested"))
            {
                return;
            }

            whsInfo = sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), current_SourceLocationCode);
            

            isSourceLocationLocked = (bool)sboProviderService.InvokeMethodWithDbConnection<object>(false, null, null,
             delegate(SboConnection conn, object[] paramlist)
             {
                 PmxOseStorageLocationProvider prov = new PmxOseStorageLocationProvider(conn);
                 return prov.IsLocationLocked(current_SourceLocationCode);
             });


            //check if picklocation is locked
            if (isSourceLocationLocked)
            {
                initialErrorKey = "MSG_ERROR_NO_PICK_LOCATION_FOUND_FOR_REPLENISH";
                goto Step_SelectMoveOrder;
            }

            isDestinationLocationLocked = (bool)sboProviderService.InvokeMethodWithDbConnection<object>(false, null, null,
               delegate(SboConnection conn, object[] paramlist)
               {
                   PmxOseStorageLocationProvider prov = new PmxOseStorageLocationProvider(conn);
                   return isDestinationLocationLocked = prov.IsLocationLocked(current_DestinationLocationCode);
               });

            //check if destinationlocations are locked      
            if (isDestinationLocationLocked)
            {
                initialErrorKey = "MSG_ERROR_DESTINATION_LOCATION_LOCKED";
                goto Step_SelectMoveOrder;
            }

            #endregion SELECT MOVE ORDER SCREEN

        Step_IdentifyFullPalletPickingStart:

            moveItems = new Collection<MoveItemInfo>();

            //there is no SSCC to scan move to select picking location screen
            if (string.IsNullOrEmpty(current_SSCC))
            {
                goto Step_SelectPickingLocationStart;
            }

            recieved_SSCC = null;

            initialErrorKey = null;

        Step_IdentifyFullPalletPicking:

            #region IDENTIFY FULL PALLET PICKING SCREEN

            isFullMonoPallet = (bool)sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
              delegate(PmxDbConnection conn, object[] paramlist)
              {
                  PmxInventoryProvider prov = new PmxInventoryProvider(conn);
                  return prov.IsFullMonoPallet(current_LUID.Value, current_ItemCode, current_OpenQuantity);
              });

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Picking.IIdentifyFullPalletPickingScreen),
                               DefaultCultureInfo.Get(), BuildParamCollection(
                                   "TitleKey", "MSG_TITLE_IDENTIFY_SSCC",
                                   "InitialErrorKey", initialErrorKey,
                                   "SSCC", current_SSCC,
                                   "ItemCode", current_ItemCode,
                                   "ItemDescription", current_ItemDescription,
                                   "Uom", current_Uom,
                                   "BatchNumber", current_Batchnumber,
                                  "BestBeforeDate", current_BestBeforeDate,
                                  "LocationCode", current_SourceLocationCode,
                                  "StillToPick", current_OpenQuantity,
                                  "ShowFullNextItemData", true//isFullMonoPallet
                                   ));

            msg = WaitForMessage();

            if (msg.Name.EndsWith("BackRequested"))
            {
                goto Step_SelectMoveOrder;
            }

            if (msg.Name.EndsWith("SSCCEntered"))
            {
                recieved_SSCC = ExtractParameter<string>(msg.Parameters, "sscc");

                //check if SSCC is valid
                if (string.IsNullOrEmpty(recieved_SSCC) || recieved_SSCC != current_SSCC)
                {
                    initialErrorKey = "MSG_ERROR_INCORRECT_SSCC_TO_PICK";
                    goto Step_IdentifyFullPalletPicking;
                }

                //this.LocationCode = this.NeededLocation;
                m_sourceLocationCode = current_SourceLocationCode;


                if (isFullMonoPallet)
                {
                    #region IS full Mono Pallet functionality

                    query = BuildQuery.GetProductOnSSCC(current_SSCC);
                    dsProductsOnSSCC = sboProviderService.RunView(false, null, null, query);

                    //set ItemCodes
                    itemCodes = new Collection<string>();
                    if (GetNumberOfRows(dsProductsOnSSCC) > 0)
                    {
                        foreach (DataRow row in dsProductsOnSSCC.Tables[0].Rows)
                        {
                            itemCodes.Add(row[SboItemTable.Columns.ItemCode.NAME] as string);
                        }
                    }

                    itemInfos = new Collection<PmxItemInfo>();

                    sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null,
                      delegate(PmxDbConnection conn, object[] paramlist)
                      {
                          PmxItemAllConnectionsProvider prov = new PmxItemAllConnectionsProvider(conn);
                          itemInfos = prov.GetItemInfos(itemCodes);
                          return null;
                      });


                    if (GetNumberOfRows(dsProductsOnSSCC) > 0)
                    {
                        foreach (DataRow row in dsProductsOnSSCC.Tables[0].Rows)
                        {
                            MoveItemInfo moveItemInfo = new MoveItemInfo();

                            moveItemInfo.ItemTransactionalInfoKey = Convertor.ConvertToInt32Nullable(row[PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME]);
                            moveItemInfo.DestinationLogisticUnitIdentKey = Convertor.ConvertToInt32Nullable(row[PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME]);
                            moveItemInfo.SourceLogisticUnitIdentKey = Convertor.ConvertToInt32Nullable(row[PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME]);
                            moveItemInfo.DestinationQualityStatusCode = row[PmxInventoryTotalTable.Columns.QualityStatusCode.NAME] as string;

                            moveItemInfo.ItemCode = row[SboItemTable.Columns.ItemCode.NAME] as string;

                            PmxItemInfo itemInfo = GetItemInfoFromList(itemInfos, moveItemInfo.ItemCode);

                            if (itemInfo != null)
                            {
                                moveItemInfo.QuantityPerUom = 1;
                                moveItemInfo.Uom = itemInfo.InventoryUom;
                            }

                            moveItemInfo.ItemDescription = row[SboItemTable.Columns.ItemName.NAME] as string;
                            moveItemInfo.Quantity = Convert.ToDouble(row[PmxInventoryTotalTable.Columns.Quantity.NAME]);
                            moveItemInfo.SourceLocationCode = row[PmxInventoryTotalTable.Columns.StorLocCode.NAME] as string;
                            moveItemInfo.SourceQualityStatusCode = row[PmxInventoryTotalTable.Columns.QualityStatusCode.NAME] as string;

                            //Catch weight
                            moveItemInfo.Uom2 = null;
                            moveItemInfo.QuantityUom2 = null;


                            moveItems.Add(moveItemInfo);

                        }
                    }

                    goto Step_EnterDestinationLocationStart;

                    #endregion
                }
                else
                {
                    #region Not a full mono pallet functionality

                    //further implementation
                    goto Step_SubProductFromStockStart;

                    #endregion Not a full mono pallet functionality
                }
            }


            #endregion IDENTIFY FULL PALLET PICKING SCREEN

        Step_SelectPickingLocationStart:

            initialErrorKey = null;


        Step_SelectPickingLocation:

            #region SELECT PICKING LOCATION SCREEN

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Picking.IIdentifyLocationScreen),
                   DefaultCultureInfo.Get(), BuildParamCollection(
                   "TitleKey", "MSG_TITLE_IDENTIFY_PICK_LOCATION",
                       "BatchNumber", current_Batchnumber,
                       "BestBeforeDate", current_BestBeforeDate,
                       "InitialErrorKey", initialErrorKey,
                       "ItemCode", current_ItemCode,
                       "ItemDescription", current_ItemDescription,
                       "LocationCode", current_SourceLocationCode,
                       "StillToPick", current_OpenQuantity,
                       "TotalQuantityToPick", current_OpenQuantity,
                       "ButtonKeys", new string[] {"MSG_BUTTON_SELECT_LOCATION"}
                   ));

            msg = WaitForMessage();

            if(msg.Name.EndsWith("BackRequested") )
            {
                goto Step_SelectMoveOrder;
            }
            if (msg.Name.EndsWith("LocationEntered"))
            {
                m_sourceLocationCode = ExtractParameter<string>(msg.Parameters, "LocationCode");
            }
            else if (msg.Name.EndsWith("ButtonClicked"))
            {
                #region Buttonclicked identitify pick location screen

                query = BuildQuery.GetStorageLocation(null, whsInfo.PmxWarehouseCode);
                dsSourceLocations = sboProviderService.RunView(false, null, null, query);

                if (GetNumberOfRows(dsSourceLocations).Equals(0))
                {
                    initialErrorKey = "MSG_ERROR_NO_PICK_LOCATION_FOUND_FOR_REPLENISH";
                    goto Step_SelectMoveOrder;
                }

                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectLocationScreen),
                       DefaultCultureInfo.Get(), BuildParamCollection(
                       "TitleKey", "MSG_TITLE_SELECT_LOCATION",
                       "InitialErrorKey", initialErrorKey,
                       "LocationDS", dsSourceLocations
                       ));

                msg = WaitForMessage();

                if (msg.Name.EndsWith("LocationSelected"))
                {
                    m_sourceLocationCode = ExtractParameter<string>(msg.Parameters, "locationCode");
                }
                else
                {
                    goto Step_SelectMoveOrderStart;
                }


                #endregion
            }

            if (current_SourceLocationCode != m_sourceLocationCode)
            {
                initialErrorKey = "MSG_ERROR_INCORRECT_PICK_LOCATION";
                goto Step_SelectPickingLocation;
            }



            #endregion SELECT PICKING LOCATION SCREEN


        Step_SubProductFromStockStart:
            initialErrorKey = null;

        Step_SubProductFromStock:

            m_subInitialErrorKey = initialErrorKey;

            #region SUBFLOW SELECT PRODUCT FROM STOCK

            //GETDATA
            m_itemInfo = (PmxItemInfo)sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
              delegate(PmxDbConnection conn, object[] paramlist)
              {
                  PmxItemAllConnectionsProvider prov = new PmxItemAllConnectionsProvider(conn);
                  return prov.GetCachedItemInfo(current_ItemCode);
              });
        
            subSelectProductFromStockFlow = this.Create("SelectProductFromStockScript");
            //subItemInfo = 
            m_subBackRequested = false;

            subSelectProductFromStockFlow.BindParameters(
                this.DefaultCultureInfo,
                this.PmxOseCompany,
                new ReadOnlyBinder<string>("LocationCode", m_sourceLocationCode),
                new ReadOnlyBinder<string>("NeededSSCC", current_SSCC),
                new ReadOnlyBinder<string>("NeededBatchNumber", current_Batchnumber),
                new ReadOnlyBinder<DateTime?>("NeededBestBeforeDate", current_BestBeforeDate),
                new ReadOnlyBinder<string>("NeededItemCode", current_ItemCode),
                new ReadOnlyBinder<int?>("NeededItemTransactionalInfoKey", current_ItemTransactionalInfoKey),
                new ReadOnlyBinder<string>("NeededQualityStatusCode", current_QualityStatusCode),
                new ReadWriteBinder<string>("InitialErrorKey", this, "m_subInitialErrorKey"),
                new ReadWriteBinder<PmxItemInfo>("ItemInfo", this, "m_itemInfo"),
                new ReadWriteBinder<string>("ItemDescription", this, "current_ItemDescription"),
                new ReadWriteBinder<string>("ItemCode", this, "m_itemCode"),
                new ReadWriteBinder<int?>("ItemTransactionalInfoKey", this, "m_itemTransactionalInfoKey"),
                new ReadWriteBinder<string>("BatchNumber", this, "m_batchNumber"),
                new ReadWriteBinder<DateTime?>("BestBeforeDate", this, "m_bestBeforeDate"),
                new ReadWriteBinder<string>("QualityStatusCode", this, "m_qualityStatusCode"),
                new ReadWriteBinder<string>("SSCC", this, "m_SSCC"),
                new ReadWriteBinder<double>("Quantity", this, "m_maximumQuantity"),
                new ReadWriteBinder<string>("ItemInventoryUom", this, "m_itemInventoryUom"),
                new ReadWriteBinder<bool>("BackRequested", this, "m_subBackRequested"));


            base.StartSubFlowAndWait(subSelectProductFromStockFlow, true);


            if (m_subInitialErrorKey != null)
            {
                initialErrorKey = m_subInitialErrorKey;
                goto Step_SelectPickingLocation;
            }

            if (m_subBackRequested) goto Step_SelectMoveOrderStart;

            maximumQuantityForLocation = (double)sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
              delegate(PmxDbConnection conn, object[] paramlist)
              {
                  PmxInventoryProvider prov = new PmxInventoryProvider(conn);
                  return prov.GetMaximumQuantityForLocation(current_DestinationLocationCode, current_ItemCode);
              });


            //Take the quantity from stock instead of the open quantity of the order
            m_maximumQuantity = Math.Min(m_maximumQuantity, maximumQuantityForLocation);

            if (m_maximumQuantity == 0)
            {
                initialErrorKey = "MSG_ERROR_CANNOT_REPLENISH";
                goto Step_IdentifyFullPalletPicking;
            }



            #endregion SUBFLOW SELECT PRODUCT FROM STOCK

        Step_EnterQuantityWithInfoStart:

            initialErrorKey = null;

        Step_EnterQuantityWithInfo:

            #region ENTER QUANTITY WITH INFO SCREEN


            forceSerial = false;
            useRange = false;
            listOfSerialNumbers = null;
            m_subFlowListOfSerialNumbers = null;

            string[] buttonKeys = null;

            if (m_itemInfo.TrackLocationSerialNumber || m_itemInfo.HasSAPSerialNumber || m_itemInfo.HasSerialNumber)
            {

                buttonKeys = new string[2] { "MSG_BUTTON_SERIAL_NUMBER", "MSG_BUTTON_SERIAL_NUMBER_RANGE" };

                askForSerialNumbers = m_itemInfo.TrackLocationSerialNumber;

                if (!askForSerialNumbers)
                {
                    EnterSerialNumbers enterSerialNumber = sboProviderService.InvokeMethodWithDbConnection<EnterSerialNumbers>(false, false, null, null,
                        delegate(PmxDbConnection conn, object[] parameters)
                        {
                            PmxSerialNumberProvider prov = new PmxSerialNumberProvider(conn);
                            return prov.NeedToEnterSerialNumbers(whsInfo.PmxWarehouseCode, whsInfo.PmxWarehouseCode, current_SourceLocationCode, current_DestinationLocationCode, new Collection<string>() { m_itemInfo.ItemCode });
                        });

                    if (enterSerialNumber == EnterSerialNumbers.MustEnter)
                    {
                        askForSerialNumbers = true;
                    }
                    if (enterSerialNumber == EnterSerialNumbers.CanEnter)
                    {
                        askForSerialNumbers = true;
                        buttonKeys = new string[3] { "MSG_BUTTON_SERIAL_NUMBER", "MSG_BUTTON_SERIAL_NUMBER_RANGE", "MSG_BUTTON_NO_SERIAL_NUMBER" };
                    }

                }
            }



            if (askForSerialNumbers)
            {
                #region SERIAL NUMBERS

                forceSerial = true;
                //show screen to ask user what to do.

                ToolBarButtons toolBarButtons = ToolBarButtons.Back;

                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "TitleKey", "MSG_TITLE_ENTER_QUANTITY_SERIAL_NUMBER_TASK",
                        "AllowedButtons", buttonKeys,
                        "EnabledToolBarButtons", toolBarButtons
                        ));
                msg = WaitForMessage();
                initialErrorKey = null;
                if (msg.Name.EndsWith(".ButtonClicked"))
                {
                    useRange = ExtractParameter<string>(msg.Parameters, "buttonKey") == "MSG_BUTTON_SERIAL_NUMBER_RANGE";

                    //no serial number 
                    forceSerial = ExtractParameter<string>(msg.Parameters, "buttonKey") != "MSG_BUTTON_NO_SERIAL_NUMBER";

                }

                if (msg.Name.EndsWith(".BackRequested"))
                {
                    goto Step_SelectMoveOrder;
                }


                WorkflowInstanceBase subEnterQuantityFlow = this.Create("EnterQuantityScript");
                SubBackRequested = false;
                subEnterQuantityFlow.BindParameters(
                    this.DefaultCultureInfo,
                    this.PmxOseCompany,
                    new ReadOnlyBinder<string>("TitleKey", "MSG_TITLE_ENTER_QUANTITY"),
                    new ReadOnlyBinder<PmxItemInfo>("ItemInfo", m_itemInfo),
                    new ReadOnlyBinder<int?>("ItriKey", m_itemTransactionalInfoKey),
                    new ReadOnlyBinder<string>("BatchNumber", m_batchNumber),
                    new ReadOnlyBinder<DateTime?>("BestBeforeDate", m_bestBeforeDate),
                    new ReadOnlyBinder<string>("SSCC", m_SSCC),

                    new ReadOnlyBinder<bool>("AllowZero", false),
                    new ReadOnlyBinder<double>("InitialQuantity", 0.0),
                    new ReadOnlyBinder<double>("MaximumQuantity", m_maximumQuantity),
                    new ReadOnlyBinder<string>("UOM", m_itemInfo.InventoryUom),
                    new ReadOnlyBinder<int>("NumberOfDigitsAfterDecimal", (m_itemInfo == null) ? 0 : m_itemInfo.UomDecimals),
                    new ReadOnlyBinder<double>("InnerQuantity", 0.0),
                    new ReadOnlyBinder<string>("InnerUOM", null),

                    new ReadOnlyBinder<string>("DocNum", MODocEntry.HasValue? MODocEntry.Value.ToString(): null),
                    new ReadOnlyBinder<double>("StillToQuantity", m_maximumQuantity),
                    new ReadOnlyBinder<string>("MessageKeyStillQuantity", "MSG_LBL_MAXIMUM_QUANTITY"),

                    new ReadOnlyBinder<bool>("ForceInputSerialNumbers", forceSerial),
                    new ReadOnlyBinder<string>("ObjectType", Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxObjectTypes.PmxMoveOrder),
                    new ReadOnlyBinder<int?>("DocEntry", MODocEntry),
                    new ReadOnlyBinder<bool>("UseSerialNumberRange", useRange),
                    new ReadOnlyBinder<string>("LocationCode", m_sourceLocationCode),
                    new ReadOnlyBinder<string>("DestinationLocationCode", current_DestinationLocationCode),

                    new ReadOnlyBinder<int?>("LUID", current_LUID),

                    new ReadWriteBinder<bool>("BackRequested", this, "SubBackRequested"),
                    new ReadWriteBinder<double>("QuantityEntered", this, "SubQuantity"),
                    new ReadWriteBinder<Collection<string>>("ListOfSerialNumbers", this, "m_subFlowListOfSerialNumbers")
                    );

                base.StartSubFlowAndWait(subEnterQuantityFlow, true);
                listOfSerialNumbers = m_subFlowListOfSerialNumbers;

                if (SubBackRequested) goto Step_SelectMoveOrder;

                recieved_Quantity = SubQuantity;

                #endregion

            }
            else
            {

                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityWithItemInfoScreen),
                                   DefaultCultureInfo.Get(), BuildParamCollection(
                                   "BatchNumber", current_Batchnumber, // is null
                                   "BestBeforeDate", current_BestBeforeDate,// is null
                                   "InitialQuantity", current_OpenQuantity,
                                   "ItemCode", current_ItemCode, //cant get it in
                                   "ItemDescription", current_ItemDescription,
                                   "MaximumQuantity", m_maximumQuantity,
                                   "NumberOfDigitsAfterDecimal", m_itemInfo.UomDecimals,
                                   "SSCC", recieved_SSCC, //null
                                   "StillToQuantity", current_OpenQuantity,
                                   "MessageKeyStillQuantity", "MSG_LBL_STILL_TO_MOVE",
                                   "TitleKey", "MSG_TITLE_ENTER_QUANTITY"
                                   ));

                msg = WaitForMessage();

                //inverted if
                if (!msg.Name.EndsWith("QuantityEntered"))
                {
                    goto Step_SelectMoveOrder;
                }

                recieved_Quantity = ExtractParameter<double>(msg.Parameters, "Quantity");

            }

            query = BuildQuery.GetLuidFromStock(recieved_SSCC, m_sourceLocationCode, current_ItemCode, current_Batchnumber, current_BestBeforeDate, current_QualityStatusCode);
            dsLuidFromStock = sboProviderService.RunView(false, null, null, query);

            if (GetNumberOfRows(dsLuidFromStock) > 0)
            {
                double quanityToMove = recieved_Quantity;
                foreach (DataRow row in dsLuidFromStock.Tables[0].Rows)
                {
                    MoveItemInfo moveItem = new MoveItemInfo();

                    moveItem.ItemTransactionalInfoKey = current_ItemTransactionalInfoKey;

                    moveItem.DestinationLogisticUnitIdentKey = null;
                    moveItem.DestinationQualityStatusCode = current_QualityStatusCode;
                    moveItem.ItemCode = current_ItemCode;
                    moveItem.ItemDescription = current_ItemDescription;
                    
                    moveItem.QuantityPerUom = 1;//correct qty?
                    moveItem.Uom = m_itemInfo.InventoryUom;

                    moveItem.Quantity = Math.Min(quanityToMove, Convert.ToDouble(row[PmxInventoryTotalTable.Columns.Quantity.NAME]));

                    //Catch weight
                    moveItem.Uom2 = null;
                    moveItem.QuantityUom2 = null;

                    //reduce the quantity to be moved
                    quanityToMove -= moveItem.Quantity;

                    moveItem.SourceLocationCode = m_sourceLocationCode;
                    moveItem.SourceLogisticUnitIdentKey = Convertor.ConvertToInt32Nullable(row[PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME]);
                    moveItem.SourceQualityStatusCode = current_QualityStatusCode;

                    if (listOfSerialNumbers != null &&
                        listOfSerialNumbers.Count > 0)
                    {

                        Collection<string> tempListOfSerialNumbers = new Collection<string>();

                        for (int iLoop = 0; iLoop < moveItem.Quantity; iLoop++)
                        {
                            //add to temp list
                            tempListOfSerialNumbers.Add(listOfSerialNumbers[iLoop]);
                        }

                        moveItem.ListOfSerialNumbers = tempListOfSerialNumbers;

                        foreach (string serialNumber in tempListOfSerialNumbers)
                        {
                            //remove from main list
                            listOfSerialNumbers.Remove(serialNumber);
                        }

                    }


                    moveItems.Add(moveItem);
                    if (quanityToMove <= 0)
                    {
                        //no more quantity to move, so we break
                        break;
                    }
                }//foreach

            }



            #endregion  ENTER QUANTITY WITH INFO SCREEN

        Step_EnterDestinationLocationStart:

            initialErrorKey = null;
            bool newSSCC = !isFullMonoPallet && m_itemInfo.TrackLocationSerialNumber;
            int? dstLuid = null;
            string targetLogCarrierCode = null;
            string destinationPmxWhsCode = null;

        Step_EnterDestinationLocation:

            #region ENTER DESTINATION LOCATION SCREEN

            string[] buttons = null;

            if (newSSCC)
            {
                buttons = new string[] { "MSG_BUTTON_SELECT_LOCATION"};
            }
            else
            {
                buttons = new string[] { "MSG_BUTTON_SELECT_LOCATION", "MSG_BUTTON_NEW_SSCC" };
            }



             session.ShowScreen( typeof( IScanStringValueScreen ),
                this.DefaultCultureInfo, BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", newSSCC ? "MSG_TITLE_SCAN_DESTINATION_LOCATION" : "MSG_TITLE_DST_LOCATION_OR_SSCC",
                    "ForceDataEntry", true,
                    "AllowedButtonKeys", buttons,
                    "AllowToGoBack", true,
                    "Information", "MSG_INFO_ENTER_DESTINATION_LOCATION",
                    "MessageAddendum", current_DestinationLocationCode

                    ));
            initialErrorKey = null;
            msg = WaitForMessage();

            if ( msg.Name == "Produmex.Foundation.SlimScreen.Interfaces.IScanStringValueScreenReply.StringScanned" )
            {
                string val = ExtractParameter<string>( msg.Parameters, "stringValue" );

                BarcodeContainer barcode = new BarcodeContainer();
                try
                {
                    barcode = barcodeController.GetScannedData( val, true, barcode );
                }
                catch
                {
                    // Do nothing
                }
                if ( !String.IsNullOrEmpty( barcode.GS1DataObject.Sscc ) )
                {
                    query = BuildQuery.GetSSCCInfo( barcode.GS1DataObject.Sscc );
                    ds = sboProviderService.RunView( true, null, null, query );
                    if (this.GetNumberOfRows(ds) > 0)
                    {
                        newSSCC = false;

                        dstLuid = Convert.ToInt32(ds.Tables[0].Rows[0][SSCCDataSet.Columns.LUID.Name]);
                        destinationLocationCode = Convert.ToString(ds.Tables[0].Rows[0][SSCCDataSet.Columns.STORAGE_LOCATION_CODE.Name]);
                        destinationPmxWhsCode = Convert.ToString(ds.Tables[0].Rows[0][SSCCDataSet.Columns.PMX_WAREHOUSE_CODE.Name]);
                    }
                    if (String.IsNullOrEmpty(destinationLocationCode))
                    {
                        initialErrorKey = "MSG_ERROR_INCORRECT_SSCC";
                        goto Step_EnterDestinationLocation;
                    }
                    else if (current_DestinationLocationCode != destinationLocationCode)
                    {
                        initialErrorKey = "MSG_ERROR_INCORRECT_LOCATION";
                        goto Step_EnterDestinationLocation;
                    }

                    goto Step_CheckCorrectDestinationLocationAndIfAllowedToStoreItems;
                }
                else
                {
                    destinationLocationCode = val;

                    if (current_DestinationLocationCode != destinationLocationCode)
                    {
                        initialErrorKey = "MSG_ERROR_INCORRECT_LOCATION";
                        goto Step_EnterDestinationLocation;
                    }

                    goto Step_CheckCorrectDestinationLocationAndIfAllowedToStoreItems;
                }
            }
            if ( msg.Name == "Produmex.Foundation.SlimScreen.Interfaces.IScanStringValueScreenReply.ButtonClicked" )
            {

                buttonKey = ExtractParameter<string>( msg.Parameters, "buttonKey" );
                if ( buttonKey == "MSG_BUTTON_SELECT_LOCATION")
                {
                    query = BuildQuery.GetPickLocation(whsInfo.PmxWarehouseCode);
                    dsDestinationLocations = sboProviderService.RunView(false, null, null, query);

                    initialErrorKey = null;

                    #region Screen Select DestinationLocation

                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectLocationScreen),
                                       DefaultCultureInfo.Get(), BuildParamCollection(
                                       "LocationDS", dsDestinationLocations,
                                       "InitialErrorKey", initialErrorKey,
                                       "TitleKey", "MSG_TITLE_SELECT_DESTINATION_LOCATION"
                                       ));

                    msg = WaitForMessage();

                    if (!msg.Name.EndsWith("LocationSelected"))
                    {
                        goto Step_EnterDestinationLocation;
                    }

                    destinationLocationCode = ExtractParameter<string>(msg.Parameters, "locationCode");

                    #endregion

                    goto Step_CheckCorrectDestinationLocationAndIfAllowedToStoreItems;
                }
                if ( buttonKey == "MSG_BUTTON_NEW_SSCC" )
                {
                    newSSCC = true;
                    goto Step_EnterDestinationLocation;
                }
            }
            if ( msg.Name == "Produmex.Foundation.SlimScreen.Interfaces.IScanStringValueScreenReply.BackRequested" )
            {
                goto Step_SelectMoveOrderStart;
            }

            goto Step_SelectMoveOrderStart;

           

            //old code



            //session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
            //                   DefaultCultureInfo.Get(), BuildParamCollection(
            //                   "AllowToGoBack", true,
            //                   "InitialErrorKey", initialErrorKey,
            //                   "MessageAddendum", current_DestinationLocationCode,
            //                   "Information", "MSG_INFO_ENTER_DESTINATION_LOCATION",
            //                   "ButtonKey", "MSG_BUTTON_SELECT_LOCATION",
            //                   "TitleKey", "MSG_TITLE_ENTER_DESTINATION_LOCATION"
            //                  ));

            //msg = WaitForMessage();

            //if (msg.Name.EndsWith("ButtonClicked"))
            //{
            //    query = BuildQuery.GetPickLocation(null);
            //    dsDestinationLocations = sboProviderService.RunView(false, null, null, query);

            //    initialErrorKey = null;

            //    #region Screen Select DestinationLocation

            //    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectLocationScreen),
            //                       DefaultCultureInfo.Get(), BuildParamCollection(
            //                       "LocationDS", dsDestinationLocations,
            //                       "InitialErrorKey", initialErrorKey,
            //                       "TitleKey", "MSG_TITLE_SELECT_DESTINATION_LOCATION"
            //                       ));

            //    msg = WaitForMessage();

            //    if (!msg.Name.EndsWith("LocationSelected"))
            //    {
            //        goto Step_EnterDestinationLocationStart;
            //    }

            //    destinationLocationCode = ExtractParameter<string>(msg.Parameters, "locationCode");

            //    #endregion
            //}
            //else if (msg.Name.EndsWith("StringEntered"))
            //{
            //    destinationLocationCode = ExtractParameter<string>(msg.Parameters, "stringValue");
            //}
            //else
            //{
            //    goto Step_SelectMoveOrderStart;
            //}


            #endregion ENTER DESTINATION LOCATION SCREEN

        Step_CheckCorrectDestinationLocationAndIfAllowedToStoreItems:

            #region CHECK CORRECT DESTINATION LOCATION AND IF ALLOWED TO STORE ITEMS

            if (destinationLocationCode != current_DestinationLocationCode)
            {
                initialErrorKey = "MSG_ERROR_INCORRECT_DESTINATION_LOCATION";
                goto Step_EnterDestinationLocation;
            }

           

            itemQuantityItriKeysBatchNumsPerItemCode = new Dictionary<string, PmxStorageLocationCanItemBeStoredInfo>();
            foreach (MoveItemInfo move in moveItems)
            {
                PmxStorageLocationCanItemBeStoredInfo entry;
                if (!itemQuantityItriKeysBatchNumsPerItemCode.TryGetValue(move.ItemCode, out entry))
                {
                    entry = new PmxStorageLocationCanItemBeStoredInfo(move.Quantity, null, null);
                    itemQuantityItriKeysBatchNumsPerItemCode.Add(move.ItemCode, entry);
                }
                else
                {
                    entry.Quantity += move.Quantity;
                }
                if (move.ItemTransactionalInfoKey.HasValue)
                {
                    if (entry.ItriKeys == null) entry.ItriKeys = new List<int>();
                    entry.ItriKeys.Add(move.ItemTransactionalInfoKey.Value);
                }
            }

            canItemBeStoredOnLocation = (bool)sboProviderService.InvokeMethodWithDbConnection<object>(false, null, null,
                        delegate(SboConnection conn, object[] paramlist)
                        {
                            PmxOseStorageLocationProvider prov = new PmxOseStorageLocationProvider(conn);
                            return prov.CanItemsBeStoredOnStorageLocation(destinationLocationCode, itemQuantityItriKeysBatchNumsPerItemCode);
                        });



            if (!canItemBeStoredOnLocation)
            {
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                                   DefaultCultureInfo.Get(), BuildParamCollection(
                                       "MessageKey", "MSG_CONFIRM_ADDING_DIFFERENT_ITEM_OR_BATCH",
                                       "TitleKey", "MSG_TITLE_CONFIRM_MOVE"
                                   ));

                msg = WaitForMessage();

                if (msg.Name.EndsWith("IDecisionScreenReply.Yes"))
                {
                    goto Step_WAProcessing;
                }
                else
                {
                    goto Step_EnterDestinationLocation;
                }
            }
            else
            {
                foreach (KeyValuePair<string, PmxStorageLocationCanItemBeStoredInfo> kv in itemQuantityItriKeysBatchNumsPerItemCode)
                {
                    if (kv.Value.Status != PmxStorageLocationCanItemBeStored.Yes)
                    {
                        // Warn user and ask to continue
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "TitleKey", "MSG_TITLE_CONFIRM_MOVE",
                                "MessageKey", "MSG_CONFIRM_ADDING_DIFFERENT_ITEM_OR_BATCH"));

                        msg = WaitForMessage();

                        if (msg.Name.EndsWith("IDecisionScreenReply.No"))
                        {
                            goto Step_WAProcessing;
                        }
                        break;
                    }
                }
            }

            #endregion CHECK CORRECT DESTINATION LOCATION AND IF ALLOWED TO STORE ITEMS

        Step_WAProcessing:

            #region WA PROCESSING

            //Check if move is to warehouse automation location
            if (listOfLocationsForWA.Contains(destinationLocationCode))
            {
                if (newSSCC || dstLuid.HasValue)
                {
                    //This is not allowed for move to WA locations
                    initialErrorKey = "MSG_ERROR_MOVE_TO_WA_LOCATION_SHOULD_BE_WITHOUT_SSCC";

                    goto Step_EnterDestinationLocationStart;
                }


                SubBackRequested = false;
                m_waProcessingSubFlow.BindParameters(
                    this.DefaultCultureInfo,
                    this.PmxOseCompany,
                    new ReadOnlyBinder<List<MoveItemInfo>>("MoveItemInfoList", new List<MoveItemInfo>(moveItems)),
                    new ReadOnlyBinder<string>("DestinationLocationCode", destinationLocationCode),


                    new ReadWriteBinder<bool>("BackRequested", this, "SubBackRequested"),
                    new ReadWriteBinder<string>("ActualDestinationLocationCode", this, "m_subFlowActualDestinationLocationCode"),
                    new ReadWriteBinder<List<MoveItemInfo>>("ActualMoveItemInfoList", this, "m_subFlowActualMoveItemInfoList")
                );

                this.StartSubFlowAndWait(m_waProcessingSubFlow, true);


                if (SubBackRequested)
                {
                    goto Step_EnterDestinationLocationStart;
                }

                moveItems = new Collection<MoveItemInfo>(m_subFlowActualMoveItemInfoList);
                destinationLocationCode = m_subFlowActualDestinationLocationCode;


            }

            goto Step_ExecuteMove;

            #endregion


        Step_ExecuteMove:

            #region EXECUTE MOVE SHOW MOVE CONFIRMED SCREEN


            try
            {
                sboProviderService.InvokeMethodWithDbConnection<object>(false, null, null,
                             delegate(SboConnection conn, object[] paramlist)
                             {
								 if (moveItems.Count == 0)
								 {
									 //This should not happen!!
									 s_log.ErrorFormat("Replenishment: moveItems list is empty for Move order {0}, Destination {1}", MODocEntry.Value, destinationLocationCode);
								 }

                                 foreach (MoveItemInfo move in moveItems)
                                 {
                                     move.DestinationLocationCode = destinationLocationCode;
                                     move.BaseEntry = MODocEntry.Value;
                                     move.BaseType = Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxObjectTypes.PmxMoveOrder;

                                     PmxItemInfo itemInfo = GetItemInfoFromList(itemInfos, move.ItemCode);
                                     if (itemInfo == null)
                                     {
                                         PmxItemAllConnectionsProvider itemProv = new PmxItemAllConnectionsProvider(conn);
                                         itemInfo = itemProv.GetCachedItemInfo(move.ItemCode);
                                     }

                                     if (itemInfo.IsLogisticCarrier)
                                     {
                                         move.BaseLine = -1;
                                         if (!newSSCC)
                                         {
                                             //check if log carrier is in move order doc
                                             query = BuildQuery.GetMoveOrderLineOfLogCarrier(MODocEntry.Value, move.ItemCode, move.SourceLogisticUnitIdentKey.Value, sboProviderService.GetDbTool());
                                             DataSet temp_ds = dsMoveOrders = sboProviderService.RunView(false, null, null, query);

                                             if (GetNumberOfRows(temp_ds) > 0)
                                             {
                                                 move.BaseLine = Convertor.ConvertToInt32Nullable(temp_ds.Tables[0].Rows[0][PmxMoveOrderLineTable.Columns.LineNum.NAME]);
                                             }
                                         }
                                     }                                   
                                     else
                                     {
                                         move.BaseLine = MOLineID.Value;
                                     }

                                 }

                                 if (newSSCC)
                                 {
                                     PmxLogisticUnitIDProvider luidProv = new PmxLogisticUnitIDProvider(conn);
                                     dstLuid = luidProv.GenerateNewLogisticUnit(null);                                   
                                 }

                                 if (dstLuid.HasValue)
                                 {
                                     foreach (MoveItemInfo itemInfo in moveItems)
                                     {
                                         if (!itemInfo.DestinationLogisticUnitIdentKey.HasValue)
                                         {
                                             itemInfo.DestinationLogisticUnitIdentKey = dstLuid;
                                         }
                                         itemInfo.DestinationPmxWhsCode = whsInfo.PmxWarehouseCode;
                                     }
                                 }

                                 PmxMoveProvider moveProv = new PmxMoveProvider(conn);
                                 moveProv.MakeMoveFromReplenishOrder(moveItems);

                                 return null;
                             });

                //Inform the user that the move is successfull and confirmed
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "ShowButton", true,
                                "MessageKey", "MSG_ITEMS_MOVED"));

                msg = WaitForMessage();
            }
            catch (Exception ex)
            {
                s_log.Error("", ex);
                if (!(ex is ProdumexException) || ((ProdumexException)ex).Error == null)
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", ExceptionHelper.Format( ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
                            "NoTranslationOfMessageKey", true,
                            "IsError", true,
                            "ShowButton", true));
                    WaitForMessage();
                }
                else
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "PmxError", ((ProdumexException)ex).Error,
                            "ShowButton", true));
                    WaitForMessage();
                }

                //retry?
                //ask user if he wants to retry
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "TitleKey", "MSG_TITLE_RETRY_UPDATE",
                        "MessageKey", "MSG_QUESTION_AN_ERROR_OCCURED_WANT_RETRY"));
                msg = WaitForMessage();

                if (msg.Name.ToLower().EndsWith(".yes"))
                {
                    goto Step_ExecuteMove;
                }

                goto Step_SelectMoveOrderStart;

            }


            // Print LABEL
            if (newSSCC)
            {
                bool proceed = false;
                while (!proceed)
                {
                    numberOfReports = 0;

                    try
                    {
                        //trigger event
                        numberOfReports += sboProviderService.InvokeMethodWithDbConnection<int>(false, true, null, null, delegate(PmxDbConnection conn, object[] parameters)
                        {
                            PmxPrintReportProvider printProv = new PmxPrintReportProvider(conn);
                            return printProv.PrintReportsForEvent(PmxPrintReportEventType.WarehouseCreatedLogisticUnit, current_DestinationLocationCode, session.DeviceInstanceId, dstLuid.Value, out askReprint);
                        });
                    }
                    catch (Produmex.Foundation.ProdumexException ex)
                    {
                        s_log.Error(ex.ToString());
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "InitialErrorKey", initialErrorKey,
                                "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                                "MessageAddendum", ex.Message,
                                "PmxError", ex.Error,
                                "ShowButton", true));

                        msg = WaitForMessage();
                    }
                    catch (Exception ex)
                    {
                        s_log.Error(ex.ToString());
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "InitialErrorKey", initialErrorKey,
                                "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                                "MessageAddendum", ex.Message,
                                "PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
                                "ShowButton", true));

                        msg = WaitForMessage();
                    }

                    if (numberOfReports > 0 && askReprint)
                    {
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "MessageKey", "MSG_QUESTION_REPRINT_LABEL",
                                "TitleKey", "MSG_TITLE_REPRINT_LABEL"
                                ));

                        msg = WaitForMessage();

                        if (msg.Name.EndsWith(".No"))
                        {
                            proceed = true;
                        }

                    }
                    else
                    {
                        proceed = true;
                    }

                }


            }




            goto Step_SelectMoveOrderStart;

            #endregion EXECUTE MOVE SHOW MOVE CONFIRMED SCREEN


            return;
        }



        #endregion

        #region HELPERS


        /// <summary>
        /// Gets the number of rows.
        /// </summary>
        /// <param name="ds">The ds.</param>
        /// <returns></returns>
        private int GetNumberOfRows(DataSet ds)
        {
            if (ds != null && ds.Tables.Count > 0)
            {
                return ds.Tables[0].Rows.Count;
            }
            return 0;
        }


        /// <summary>
        /// Executes the get item info.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxItemInfo ExecuteGetItemInfo(PmxDbConnection conn, object[] parameters)
        {
            PmxItemAllConnectionsProvider prov = new PmxItemAllConnectionsProvider(conn);
            return prov.GetCachedItemInfo((string)parameters[0]);
        }


        private static PmxOseWarehouseInfo ExecuteGetPmxWarehouseCode(PmxDbConnection conn, object[] parameters)
        {
            PmxOseProvider oseProv = new PmxOseProvider(conn);
            return oseProv.GetPmxWarehouseInfo((string)parameters[0]);
        }

        private PmxItemInfo GetItemInfoFromList(Collection<PmxItemInfo> itemInfos, string itemCode)
        {
            if (itemInfos != null)
            {
                foreach (PmxItemInfo itemInfo in itemInfos)
                {
                    if (itemInfo.ItemCode == itemCode)
                    {
                        return itemInfo;
                    }
                }
            }
            return null;
        }

        #endregion
    }

    #region BUILD QUERY

    /// <summary>
    /// Static class grouping our native queries
    /// </summary>
    internal static class BuildQuery
    {

        
            /// <summary>
            /// Command for getting all logistic carriers order by name
            /// </summary>
            /// <param name="onlyInStock">if set to <c>true</c> the system will get the logistic carriers that are in stock and have no products on it (LUID IS NULL)</param>
            /// <param name="filterOutFrozenItems">if set to <c>true</c> [filter out frozen items].</param>
            /// <returns>
            /// Command for getting all logistic carriers order by name
            /// </returns>
        public static string GetLogisticCarriersOrderByName( bool onlyInStock, bool filterOutFrozenItems, DbTool dbTool )
        {
            DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();

            strBuilder.Append("SELECT DISTINCT ");
            SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, SboItemTable.NAME, SboItemTable.Columns.ItemCode.NAME, ProductDataSet.Columns.PRODUCT_CODE.Name );

            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, SboItemTable.NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ProductDataSet.Columns.PRODUCT_DESCRIPTION.Name );

            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, SboItemTable.NAME, SboItemTable.Columns.CodeBars.NAME, ProductDataSet.Columns.GTIN.Name );

            strBuilder.Append(", ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.IsLogisticCarrier.NAME));

            strBuilder.Append("\r\n FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));

            if (onlyInStock)
            {
                strBuilder.Append("\r\n INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(" ON ", strBuilder.AsIdentifier(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME));
                strBuilder.Append(" = ", strBuilder.AsIdentifier(SboItemTable.NAME, SboItemTable.Columns.ItemCode.NAME));
                strBuilder.Append(" AND ", strBuilder.AsIdentifier(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME));
                strBuilder.Append(" IS NULL ");
            }

            strBuilder.Append("\r\n WHERE ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.InventoryItem.NAME));
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

            strBuilder.Append(" AND ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.IsLogisticCarrier.NAME));
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

            if (filterOutFrozenItems)
            {
                Produmex.Sbo.Logex.Data.SboCommands.GetFrozenItemFilterSQLWherePart(strBuilder, " AND ");
            }

            strBuilder.Append("\r\n ORDER BY ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.CustomItemDescr.NAME));

            return strBuilder.ToString();
        }

        



        internal static string GetSSCCInfo(string sscc)
        {
            StringBuilder strBuilder = new StringBuilder();

            strBuilder.AppendFormat("SELECT DISTINCT {0}",
                SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME, SSCCDataSet.Columns.LUID.Name));
            strBuilder.AppendFormat(", {0}",
                SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME, SSCCDataSet.Columns.SSCC.Name));
            strBuilder.AppendFormat(", {0}",
                SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME, SSCCDataSet.Columns.STORAGE_LOCATION_CODE.Name));
            strBuilder.AppendFormat(", {0}",
                SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, SSCCDataSet.Columns.PMX_WAREHOUSE_CODE.Name));

            strBuilder.AppendFormat(", {0}",
                SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, SSCCDataSet.Columns.PMX_WAREHOUSE_NAME.Name));


            strBuilder.AppendFormat("\r\n FROM {0}",
                SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));

            strBuilder.AppendFormat("\r\n INNER JOIN {0}",
                SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME));

            strBuilder.AppendFormat(" ON {0} = {1}",
                SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME),
                SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME));

            strBuilder.AppendFormat("\r\n INNER JOIN {0}",
            SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));

            strBuilder.AppendFormat(" ON {0} = {1}",
                SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME),
                SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME));

            strBuilder.AppendFormat("\r\n INNER JOIN {0}",
                SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
            strBuilder.AppendFormat(" ON {0} = {1}",
                SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME),
                SqlCommandHelper.EscapeTableColumnName(PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME));




            strBuilder.AppendFormat("\r\n WHERE {0} = {1}",
                SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME),
                SqlCommandHelper.FormatAndEscapeColumnValue(sscc, CultureInfo.InvariantCulture));

            return strBuilder.ToString();
        }


        private static SboBooleanConvertor m_sboBooleanConvertor = null;
        private static SboBooleanConvertor BooleanConvertor
        {
            get
            {
                if (m_sboBooleanConvertor == null)
                {
                    m_sboBooleanConvertor = new SboBooleanConvertor();
                }
                return m_sboBooleanConvertor;
            }
        }

        private static PmxMoveOrderTypeConvertor m_pmxMoveOrderTypeConvertor = null;

        /// <summary>
        /// Gets the boolean convertor.
        /// </summary>
        /// <value>The boolean convertor.</value>
        public static PmxMoveOrderTypeConvertor MoveOrderTypeConvertor
        {
            get
            {
                if (m_pmxMoveOrderTypeConvertor == null)
                {
                    m_pmxMoveOrderTypeConvertor = new PmxMoveOrderTypeConvertor();
                }
                return m_pmxMoveOrderTypeConvertor;
            }
        }

        /// <summary>
        /// Gets the move orders orders by priority due date.
        /// </summary>
        /// <param name="sscc">The SSCC.</param>
        /// <param name="sourceStorageLocationCode">The source storage location code.</param>
        /// <param name="itemCode">The item code.</param>
        /// <param name="batchNumber">The batch number.</param>
        /// <param name="moType">Type of the mo.</param>
        /// <param name="noLogisticCarriers">if set to <c>true</c> [no logistic carriers].</param>
        /// <param name="packagingUnit">The packaging unit.</param>
        /// <returns>Sql statement to get the data</returns>
        public static string GetMoveOrdersOrdersByPriorityDueDate( string sscc, string sourceStorageLocationCode, string itemCode, string batchNumber, PmxMoveOrderType moType, bool noLogisticCarriers, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit packagingUnit, string pmxWarehouseCode, string deviceID, DbTool dbTool )
        {
            DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();
            strBuilder.Append("SELECT ");

            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxMoveOrderTable.NAME, PmxMoveOrderTable.Columns.DocEntry.NAME, MoveOrderDataSet.Columns.MO_DOC_ENTRY.Name);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.SourceStorageLocationCode.NAME, MoveOrderDataSet.Columns.SOURCE_STORAGE_LOCATION.Name);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.DestinationStorageLocationCode.NAME, MoveOrderDataSet.Columns.DESTINATION_STORAGE_LOCATION.Name);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.ItemCode.NAME, MoveOrderDataSet.Columns.ITEMCODE.Name);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.ItemDescription.NAME, MoveOrderDataSet.Columns.ITEMDESCRIPTION.Name);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.OpenQty.NAME, MoveOrderDataSet.Columns.OPENQTY.Name);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.LineNum.NAME, MoveOrderDataSet.Columns.LINEID.Name);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.QualityStatusCode.NAME, MoveOrderDataSet.Columns.SOURCE_QUALITY_STATUS_CODE.Name);
            strBuilder.Append(", ");
            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME, MoveOrderDataSet.Columns.BATCHNUMBER.Name));
            strBuilder.Append(", ");
            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber2.NAME, MoveOrderDataSet.Columns.BATCHNUMBER2.Name));
            strBuilder.Append(", ");
            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME, MoveOrderDataSet.Columns.SSCC.Name));
            strBuilder.Append(", ");
            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.SourceLogisticUnitIdentificationKey.NAME, MoveOrderDataSet.Columns.LUID.Name));
            strBuilder.Append(", ");
            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME, MoveOrderDataSet.Columns.BESTBEFOREDATE.Name));
            strBuilder.Append(", ");
            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.ItemTransactionalInfoKey.NAME, MoveOrderDataSet.Columns.ITEMTRANSACTIONALINFOKEY.Name));
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxMoveOrderTable.NAME, PmxMoveOrderTable.Columns.MoveLogUnitIn1Time.NAME, MoveOrderDataSet.Columns.MOVELUIDIN1TIME.Name);
            strBuilder.Append(", ");
            Produmex.Sbo.Logex.Data.SboCommands.AddBarcodeUsingPackagingUnit(strBuilder, packagingUnit, MoveOrderDataSet.Columns.GTIN.Name);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.Uom.NAME, MoveOrderDataSet.Columns.UOM.Name);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.QuantityPerUom.NAME, MoveOrderDataSet.Columns.QUANTITY_PER_UOM.Name);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, SboItemTable.NAME, SboItemTable.Columns.InventoryUom.NAME, MoveOrderDataSet.Columns.INVENTORY_UOM.Name);
            strBuilder.Append(", ");
            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.OpenQtyUom2.NAME, MoveOrderDataSet.Columns.OPENQTYUOM2.Name));
            strBuilder.Append(", ");
            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.DestinationLogisticUnitIdentificationKey.NAME, MoveOrderDataSet.Columns.DESTINATION_LUID.Name));

            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxMoveOrderTable.NAME, PmxMoveOrderTable.Columns.FromPmxWhsCode.NAME, MoveOrderDataSet.Columns.SOURCE_WHS.Name);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxMoveOrderTable.NAME, PmxMoveOrderTable.Columns.ToPmxWhsCode.NAME, MoveOrderDataSet.Columns.DESTINATION_WHS.Name);


            strBuilder.Append(" FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxMoveOrderTable.NAME));

            strBuilder.Append(" INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxMoveOrderLineTable.NAME));
            strBuilder.Append(" ON ", strBuilder.AsIdentifier(PmxMoveOrderTable.NAME, PmxMoveOrderTable.Columns.DocEntry.NAME));
            strBuilder.Append(" = ", strBuilder.AsIdentifier(PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.DocEntry.NAME));

            strBuilder.Append(" INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
            strBuilder.Append(" ON ", strBuilder.AsIdentifier(PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.ItemCode.NAME));
            strBuilder.Append(" = ", strBuilder.AsIdentifier(SboItemTable.NAME, SboItemTable.Columns.ItemCode.NAME));

            strBuilder.Append(" LEFT OUTER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME));
            strBuilder.Append(" ON ", strBuilder.AsIdentifier(PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.SourceLogisticUnitIdentificationKey.NAME));
            strBuilder.Append(" = ", strBuilder.AsIdentifier(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME));

            strBuilder.Append(" LEFT JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
            strBuilder.Append(" ON ", strBuilder.AsIdentifier(PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.ItemTransactionalInfoKey.NAME));
            strBuilder.Append(" = ", strBuilder.AsIdentifier(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME));

			strBuilder.Append(" LEFT JOIN ");
			strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
			strBuilder.Append(" ON ", strBuilder.AsIdentifier(PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME));
			strBuilder.Append(" = ", strBuilder.AsIdentifier(PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.SourceStorageLocationCode.NAME));


            strBuilder.Append(" WHERE ", strBuilder.AsIdentifier(PmxMoveOrderTable.NAME, PmxMoveOrderTable.Columns.DocStatus.NAME));
            strBuilder.Append(" <> ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxMoveOrderStatusColumnValues.Closed.VALUE, CultureInfo.InvariantCulture));

            strBuilder.Append(" AND ", strBuilder.AsIdentifier(PmxMoveOrderTable.NAME, PmxMoveOrderTable.Columns.MoveOrderType.NAME));
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(MoveOrderTypeConvertor.ConvertBOPropertyToColumn(moType), CultureInfo.InvariantCulture));

            strBuilder.Append(" AND ", strBuilder.AsIdentifier(PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.LineStatus.NAME));
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));

            if (!String.IsNullOrEmpty(sscc))
            {
                strBuilder.Append(" AND ", strBuilder.AsIdentifier(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(sscc, CultureInfo.InvariantCulture));
            }

            if (!String.IsNullOrEmpty(sourceStorageLocationCode))
            {
                strBuilder.Append(" AND ", strBuilder.AsIdentifier(PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.SourceStorageLocationCode.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(sourceStorageLocationCode, CultureInfo.InvariantCulture));
            }

            if (!String.IsNullOrEmpty(itemCode))
            {
                strBuilder.Append(" AND ", strBuilder.AsIdentifier(PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.ItemCode.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));
            }

            if (!String.IsNullOrEmpty(batchNumber))
            {
                strBuilder.Append(" AND ", strBuilder.AsIdentifier(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(batchNumber, CultureInfo.InvariantCulture));
            }

            if (noLogisticCarriers)
            {
                strBuilder.Append(" AND (", strBuilder.AsIdentifier(SboItemTable.NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME));
                strBuilder.Append(" <> ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));
                strBuilder.Append(" OR ", strBuilder.AsIdentifier(SboItemTable.NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME));
                strBuilder.Append(" IS NULL ) ");

            }

            if (!String.IsNullOrEmpty(pmxWarehouseCode) || !String.IsNullOrEmpty(deviceID))
            {
                strBuilder.Append(" AND ( ");
                if (!String.IsNullOrEmpty(pmxWarehouseCode))
                {
                    strBuilder.Append(strBuilder.AsIdentifier(PmxMoveOrderTable.NAME, PmxMoveOrderTable.Columns.FromPmxWhsCode.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pmxWarehouseCode, CultureInfo.InvariantCulture));
                }
                if (!String.IsNullOrEmpty(deviceID))
                {
                    if (!String.IsNullOrEmpty(pmxWarehouseCode)) strBuilder.Append(" OR ");

                    strBuilder.AppendFormat("{0} IN ( SELECT {1} FROM {2} WHERE {3} = {4} )",
                        SqlCommandHelper.EscapeTableColumnName(PmxMoveOrderTable.NAME, PmxMoveOrderTable.Columns.FromPmxWhsCode.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxThinClientWarehouseTable.NAME, PmxThinClientWarehouseTable.Columns.PmxWarehouseCode.NAME),
                        SqlCommandHelper.EscapeTableName(PmxThinClientWarehouseTable.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxThinClientWarehouseTable.NAME, PmxThinClientWarehouseTable.Columns.ParentCode.NAME),
                        SqlCommandHelper.FormatAndEscapeColumnValue(deviceID, CultureInfo.InvariantCulture));
                }
                strBuilder.Append(" )");
            }

            strBuilder.Append(" ORDER BY ", strBuilder.AsIdentifier(PmxMoveOrderTable.NAME, PmxMoveOrderTable.Columns.Priority.NAME));
            strBuilder.Append(" , ", strBuilder.AsIdentifier(PmxMoveOrderTable.NAME, PmxMoveOrderTable.Columns.DueDate.NAME));
			strBuilder.Append(" , ", strBuilder.AsIdentifier(PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Sequence.NAME));
            return strBuilder.ToString();
        }

        /// <summary>
        /// Gets all active storage locations.
        /// </summary>
        /// <returns>The sql statement to get all the location</returns>
        public static string GetAllActiveStorageLocations()
        {
            return GetStorageLocation(null, null);
        }

        /// <summary>
        /// Gets the name and the code of the  location, ordered by name.
        /// </summary>
        /// <param name="locationCode">The location code.</param>
        /// <param name="warehouseCode">The warehouse code.</param>
        /// <returns>The sql statement to get the location</returns>
        public static string GetStorageLocation(string locationCode, string warehouseCode)
        {
            StringBuilder strBuilder = new StringBuilder();
            strBuilder.Append("SELECT ");

            SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME, LocationDataSet.Columns.CODE.Name );

            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Name.NAME, LocationDataSet.Columns.DESCRIPTION.Name );

            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, LocationDataSet.Columns.WAREHOUSECODE.Name );

            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, LocationDataSet.Columns.WAREHOUSENAME.Name);


            strBuilder.Append(" FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
            strBuilder.Append(" INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);


            strBuilder.Append(" INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);

            strBuilder.Append("\r\n INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxWarehouseTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.WarehouseCode.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.SboWarehouseCode.NAME);

            strBuilder.Append("\r\n AND ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.IsManagedByProdumex.NAME);
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));



            strBuilder.Append(" WHERE ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.IsActive.NAME));
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));


            if (!string.IsNullOrEmpty(locationCode))
            {
                strBuilder.Append(" AND ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Code.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));
            }

            if (warehouseCode != null &&
                warehouseCode.Trim() != "")
            {
                strBuilder.Append(" AND ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(warehouseCode, CultureInfo.InvariantCulture));

            }

            strBuilder.Append(" ORDER BY ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Name.NAME));


            return strBuilder.ToString();
        }

        /// <summary>
        /// Command for getting all products order by name
        /// </summary>
        /// <param name="sscc">The SSCC.</param>
        /// <returns>
        /// Command for getting all products order by name
        /// </returns>
        public static string GetProductOnSSCC(string sscc)
        {
            return GetProductOnSSCC(sscc, null);
        }

        /// <summary>
        /// Command for getting all products order by name
        /// </summary>
        /// <param name="sscc">The SSCC.</param>
        /// <param name="locationCode">The location code.</param>
        /// <returns>
        /// Command for getting all products order by name
        /// </returns>
        public static string GetProductOnSSCC(string sscc, string locationCode)
        {
            StringBuilder strBuilder = new StringBuilder();
            strBuilder.Append("SELECT ");

            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.ItemCode.NAME));

            strBuilder.Append(", ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ItemName.NAME));

            strBuilder.Append(", ");
            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTable.NAME, PmxItemTable.Columns.DefaultStorageLocationOrZoneCode.NAME, PmxItemTable.Columns.DefaultStorageLocationOrZoneCode.NAME));


            strBuilder.Append(", ");
            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTable.NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME));

            strBuilder.Append(", ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));

            strBuilder.Append(", ");
            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.LogisticCarrierCode.NAME, PmxLogisticUnitIDTable.Columns.LogisticCarrierCode.NAME));

            strBuilder.Append(", ");
            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME));

            strBuilder.Append(", ");
            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME));

            strBuilder.Append(", ");
            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME));

            strBuilder.Append(", ");
            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME));

            strBuilder.Append(", ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.QualityStatusCode.NAME));

            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.InventoryUom.NAME);

            strBuilder.Append(", SUM( ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);
            strBuilder.Append(" ) AS ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.Quantity.NAME));


            strBuilder.Append(" FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));

            strBuilder.Append(" INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
            strBuilder.Append(" ON ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.Columns.ItemCode.NAME));
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.Columns.ItemCode.NAME));

            strBuilder.Append(" LEFT JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME);

            strBuilder.Append(" LEFT JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME);


            strBuilder.Append(" WHERE ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.SSCC.NAME));
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(sscc, CultureInfo.InvariantCulture));

            if (!string.IsNullOrEmpty(locationCode))
            {
                strBuilder.Append(" AND ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));
            }


            strBuilder.Append(" GROUP BY ");

            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.ItemName.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.DefaultStorageLocationOrZoneCode.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QualityStatusCode.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.InventoryUom.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.LogisticCarrierCode.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME);


            strBuilder.Append(" ORDER BY ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ItemName.NAME));
            return strBuilder.ToString();
        }


        /// <summary>
        /// Gets all the pick location.
        /// </summary>
        /// <param name="warehouseCode">The warehouse code.</param>
        /// <returns>The sql statement to get all pick locations</returns>
        public static string GetPickLocation(string warehouseCode)
        {
            StringBuilder strBuilder = new StringBuilder();
            strBuilder.Append("SELECT ");

            SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME, LocationDataSet.Columns.CODE.Name );

            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Name.NAME, LocationDataSet.Columns.DESCRIPTION.Name );

            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, LocationDataSet.Columns.WAREHOUSECODE.Name );

            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, LocationDataSet.Columns.WAREHOUSENAME.Name);


            strBuilder.Append(" FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
            strBuilder.Append(" INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);

            strBuilder.Append(" INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);


            strBuilder.Append(" WHERE ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.IsPickLoc.NAME));
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

            strBuilder.Append(" AND ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.IsActive.NAME));
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));


            if (warehouseCode != null &&
                warehouseCode.Trim() != "")
            {
                strBuilder.Append(" AND ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(warehouseCode, CultureInfo.InvariantCulture));

            }

            strBuilder.Append(" ORDER BY ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Name.NAME));


            return strBuilder.ToString();
        }


        /// <summary>
        /// Gets the quantity by luid for a specific item in stock.
        /// </summary>
        /// <param name="sscc">The SSCC.</param>
        /// <param name="location">The location.</param>
        /// <param name="itemCode">The item code.</param>
        /// <param name="batchNumber">The batch number.</param>
        /// <param name="bestBeforeDate">The best before date.</param>
        /// <param name="qualityStatusCode">The quality status code.</param>
        /// <returns></returns>
        public static string GetLuidFromStock(string sscc, string location, string itemCode, string batchNumber, DateTime? bestBeforeDate, string qualityStatusCode)
        {
            StringBuilder strBuilder = new StringBuilder();
            strBuilder.Append("SELECT ");

            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME));
            strBuilder.Append(", SUM( ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);
            strBuilder.Append(" ) AS ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.Quantity.NAME));


            strBuilder.Append(" FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));

            strBuilder.Append(" LEFT JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME);



            string operand = " WHERE ";

            if (!string.IsNullOrEmpty(sscc))
            {
                strBuilder.Append(operand);
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.SSCC.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(sscc, CultureInfo.InvariantCulture));
                operand = " AND ";
            }

            if (!string.IsNullOrEmpty(qualityStatusCode))
            {
                strBuilder.Append(operand);
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.QualityStatusCode.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(qualityStatusCode, CultureInfo.InvariantCulture));
                operand = " AND ";
            }
            if (!string.IsNullOrEmpty(location))
            {
                strBuilder.Append(operand);
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(location, CultureInfo.InvariantCulture));
                operand = " AND ";
            }

            if (!string.IsNullOrEmpty(itemCode))
            {
                strBuilder.Append(operand);
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.ItemCode.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));
                operand = " AND ";
            }

            if (!string.IsNullOrEmpty(batchNumber))
            {
                strBuilder.Append(operand);
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(batchNumber, CultureInfo.InvariantCulture));
                operand = " AND ";
            }
            else
            {
                strBuilder.Append(operand);
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME));
                strBuilder.Append(" IS NULL ");
                operand = " AND ";

            }

            if (bestBeforeDate.HasValue)
            {
                strBuilder.Append(operand);
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(bestBeforeDate.Value, CultureInfo.InvariantCulture));
                operand = " AND ";
            }
            else
            {
                strBuilder.Append(operand);
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME));
                strBuilder.Append(" IS NULL ");
                operand = " AND ";

            }

            strBuilder.Append(" GROUP BY ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME);

            strBuilder.Append(" ORDER BY ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME);

            return strBuilder.ToString();
        }

        /// <summary>
        /// Gets the move order line of log carrier.
        /// </summary>
        /// <param name="moveOrderDocEntry">The move order document entry.</param>
        /// <param name="itemCode">The item code.</param>
        /// <param name="luid">The luid.</param>
        /// <param name="dbTool">The database tool.</param>
        /// <returns></returns>
        public static string GetMoveOrderLineOfLogCarrier(int moveOrderDocEntry, string itemCode, int luid, DbTool dbTool)
        {
            DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();
            strBuilder.AppendFormat("SELECT {0}",
                SqlCommandHelper.EscapeTableColumnName(PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.LineNum.NAME));

            strBuilder.AppendFormat(" FROM {0}",
                SqlCommandHelper.EscapeTableName(PmxMoveOrderLineTable.NAME));


            strBuilder.AppendFormat("WHERE {0} = {1}",
              SqlCommandHelper.EscapeTableColumnName(PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.DocEntry.NAME),
              SqlCommandHelper.FormatAndEscapeColumnValue(moveOrderDocEntry, dbTool.CultureInfo)
              );

            strBuilder.AppendFormat("AND {0} = {1}",
             SqlCommandHelper.EscapeTableColumnName(PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.ItemCode.NAME),
             SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, dbTool.CultureInfo)
             );

            strBuilder.AppendFormat("AND {0} = {1}",
             SqlCommandHelper.EscapeTableColumnName(PmxMoveOrderLineTable.NAME, PmxMoveOrderLineTable.Columns.SourceLogisticUnitIdentificationKey.NAME),
             SqlCommandHelper.FormatAndEscapeColumnValue(luid, dbTool.CultureInfo)
             );


            return strBuilder.ToString();
        }



    #endregion
    }
}
